import pandas as pd
import pywt
import incremental_clustering

########### create some dummy data ###########

data = [[8,     3,      10],
        [8.5,   2,      9],
        [9,     2,      8],
        [8.5,   2.5,    8.5],
        [7.5,   1,      7.5],
        [7,     0,      7]]
data = pd.DataFrame(data, columns = ['tl', 'tl', 'tl'])

station_ids = ["station1", "station2", "station3"] # dummy station names

print("Data:")
print(data)


########### apply wavelet decomposition to the dummy data ###########

# specify the kind of wavelet and wavelet extension mode to be used
wavelet_mode = 'symmetric'
haarwavelet = pywt.Wavelet('haar')

# function to extract wavelet representation from a data frame column
def get_wavelet(col):
	coeffs = pywt.wavedec(col, haarwavelet, mode=wavelet_mode)
	flattened_coeffs = [item for c in coeffs for item in c] #put all the coefficients into one single array
	return pd.Series(flattened_coeffs)

# apply get_wavelet function to all columns in the data frame
wavelet_data = data.apply(get_wavelet)

print("Wavelet decomposition:")
print(wavelet_data)


########### find out the shape of the transformed data ###########

timeseries_length = len(data.index) # length of the original data
num_levels_in_decomposition = pywt.dwt_max_level(timeseries_length, haarwavelet) + 1 # total number of levels in the wavelet decomposition ( = dwt_max_level arrays of detail coefficients + the approximation coefficients)
coeff_lengths = [0] * num_levels_in_decomposition #array that will store the number of coefficients for each level

# iteratively call dwt_coeff_len to get the number of coefficients in each level
current_length = timeseries_length
for i in range(1, num_levels_in_decomposition):
    current_length = pywt.dwt_coeff_len(current_length, haarwavelet, mode=wavelet_mode)
    coeff_lengths[-i] = current_length #filling from the right side
coeff_lengths[0] = current_length # first coefficients array has the same length as the second one

print("Coefficients per level:")
print(coeff_lengths)


########### cluster! ###########

# set clustering options
timeseries_per_point = 1 # number of time series per point (relevant if panel data is used)
threshold = 10 # threshold for AWT
branchingfactor = 5 # branchingfactor for the clustering feature tree
start_with_levels = 2 # number of wavelet levels that will be used for building the clustering feature tree
max_iterations = 20 # maximum number of iterations for I-Kmeans
outputfiletag = "demo_" # tag that will be prepended to the output file names

# pass the data to the clustering library
awt = incremental_clustering.Incremental_clustering()
num_clusters = awt.cluster(wavelet_data, \
                                station_ids, 
                                coeff_lengths, \
                                len(station_ids), \
                                timeseries_per_point, \
                                len(wavelet_data.index), \
                                threshold, \
                                branchingfactor, \
                                len(coeff_lengths), \
                                start_with_levels, \
                                max_iterations, \
                                outputfiletag
)
print("Created {} clusters".format(num_clusters))